@@ -567,6 +567,12 @@ void play_sound_interface(struct sound_dev* dev, int nSamples, complex double *
 		if (dev->cr_average_count <= 0) {
 			dev->cr_correction = 0;
 		}
+		else if (dev->dev_index == t_MicPlayback && (rxMode == CWL || rxMode == CWU)) {
+			dev->cr_correction = 0;
+			dev->cr_average_fill /= dev->cr_average_count;
+			if (quisk_sound_state.verbose_sound > 1)
+				QuiskPrintf("%s:  Buffer average %5.2lf\n", dev->stream_description, dev->cr_average_fill * 100);
+		}
 		else if (dev->cr_delay > 0) {
 			dev->cr_delay--;
 			dev->cr_correction = 0;
@@ -825,6 +831,7 @@ int quisk_read_sound(void)	// Called from sound thread
 	double mic_play_volume;
 	complex double tx_mic_phase;
 	static double cwEnvelope=0;
+	static double cwCount=0;
 	static complex double tuneVector = (double)CLIP32 / CLIP16;	// Convert 16-bit to 32-bit samples
 	static struct quisk_cFilter filtInterp={NULL};
 	int key_state, is_DGT;
@@ -1032,36 +1039,41 @@ int quisk_read_sound(void)	// Called from sound thread
 		}
 		else {
 			is_cw = 0;
+			cwCount = 0;
 			cwEnvelope = 0.0;
 		}
 		tx_mic_phase = cexp(( -I * 2.0 * M_PI * quisk_tx_tune_freq) / quisk_MicPlayback.sample_rate);
-		if (is_cw) {	// Transmit CW
+		if (is_cw) {	// Transmit CW; use capture device for timing, not microphone
+			cwCount += (double)retval * quisk_MicPlayback.sample_rate / quisk_sound_state.sample_rate;
+			mic_count = 0;
 			if (QUISK_CWKEY_DOWN || quiskSpotLevel >= 0) {
-				for (i = 0; i < mic_count; i++) {
+				while (cwCount >= 1.0) {
 					if (cwEnvelope < 1.0) {
 						cwEnvelope += 1. / (quisk_MicPlayback.sample_rate * 5e-3);	// 5 milliseconds
 						if (cwEnvelope > 1.0)
 							cwEnvelope = 1.0;
 					}
 					if (quiskSpotLevel >= 0)
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
 					else
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
 					tuneVector *= tx_mic_phase;
+					cwCount -= 1;
 				}
 			}
 			else {		// key is up
-				for (i = 0; i < mic_count; i++) {
+				while (cwCount >= 1.0) {
 					if (cwEnvelope > 0.0) {
 						cwEnvelope -= 1.0 / (quisk_MicPlayback.sample_rate * 5e-3);	// 5 milliseconds
 						if (cwEnvelope < 0.0)
 							cwEnvelope = 0.0;
 					}
 					if (quiskSpotLevel >= 0)
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
 					else
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
 					tuneVector *= tx_mic_phase;
+					cwCount -= 1;
 				}
 			}
 		}
