@@ -25,7 +25,7 @@
 #include <mmreg.h>
 #include <functiondiscoverykeys_devpkey.h>
 #include <avrt.h>
-#include <Ksmedia.h>
+#include <ksmedia.h>
 #include <processthreadsapi.h>
 
 #define EXIT_ON_ERROR(hres)  \
@@ -71,7 +71,7 @@ struct dev_data_t {
 
 static LPWSTR to_pwsz(const char *str) {	// Convert UTF-8 string to Wide Character string.
 	static wchar_t wchar_buffer[256];
-	if (MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, str, -1, wchar_buffer, 256) == 0)
+	if (MultiByteToWideChar(CP_UTF8, 0, str, -1, wchar_buffer, 256) == 0)
 		return NULL;
 	return wchar_buffer;
 }
